#define POLAR_VIEW

#include <windows.h>
#include <stdio.h>
#include <GL\glut.h>
#include "..\HierarquicoCommon\Util.h"
#include "..\HierarquicoCommon\Camara.h"
#include "..\HierarquicoCommon\RobotArm.h"

/* menu options */
#define MENU_EXIT			1
#define MENU_WIREFRAME		20
#define MENU_SOLID			21
#define MENU_SHOW_AXIS		22	
#define MENU_HIDE_AXIS		23
#define MENU_RESET_CAMERA			38
#define MENU_RESET_CAMERAORG		39



struct Estado
{
	Camara* cam;
	GLenum mode;
	bool showAxis;
} estado;

struct Modelo
{
	RobotArm braco;
} modelo;

void init_estado()
{
	estado.mode = GLU_LINE;
#ifdef POLAR_VIEW
	estado.cam = new PolarCamara();
#else
	estado.cam = new EyeCamara();
#endif
	estado.showAxis = true;
}

void destroy_estado()
{
	delete estado.cam;
}

void init_modelo()
{
	// nada a fazer
}

void destroy_modelo()
{
	// nada a fazer
}

void display()
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// posicionar camara
	estado.cam->SetView();

	// sistema global de coordenadas
	drawAxis(3);

	// colocar desenho alinhado com eixo Y
	glRotatef(-90, 1, 0, 0);

	// desenhar a cena
	modelo.braco.Display(estado.mode, estado.showAxis);

	glFlush();
	glutSwapBuffers();
}

void setview()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	//glOrtho(-3.0, 3.0, -3.0, 3.0, 0.1, 20.0);
	gluPerspective(60, 1, 1, 10);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	estado.cam->SetView();
}

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
	setview();
}

#ifdef POLAR_VIEW
void reset_camera()
{
	((PolarCamara*)estado.cam)->SetCamara(6, 0, 45, -33);
}

void reset_camera_org()
{
	((PolarCamara*)estado.cam)->SetCamara(6, 0, 0, 0);
}
#else
void reset_camera()
{
	((EyeCamara*)estado.cam)->SetCamara(-0.6, -0.9, -2);
}

void reset_camera_org()
{
	((EyeCamara*)estado.cam)->SetCamara(0, 0, -3);
}
#endif

void menu(int op)
{
	switch (op)
	{
	case MENU_EXIT:
		exit(0);
		break;
	case MENU_RESET_CAMERA:
		reset_camera();
		break;
	case MENU_RESET_CAMERAORG:
		reset_camera_org();
		break;
	case MENU_WIREFRAME:
		estado.mode = GLU_LINE;
		break;
	case MENU_SOLID:
		estado.mode = GLU_FILL;
		break;
	case MENU_SHOW_AXIS:
		estado.showAxis = true;
		break;
	case MENU_HIDE_AXIS:
		estado.showAxis = false;
		break;
	}
	glutPostRedisplay();
}

void keyboard(unsigned char key, int x, int y)
{
	switch (key)
	{
	case 27:
		exit(0);
		break;
	case 'q':
	case 'Q':
		modelo.braco.IncRotBase(3);
		break;
	case 'a':
	case 'A':
		modelo.braco.DecRotBase(3);
		break;
	case 'w':
	case 'W':
		modelo.braco.IncRotSeg1(3);
		break;
	case 'S':
	case 's':
		modelo.braco.DecRotSeg1(3);
		break;
	case 'e':
	case 'E':
		modelo.braco.IncRotSeg2(3);
		break;
	case 'd':
	case 'D':
		modelo.braco.DecRotSeg2(3);
		break;
	case 'r':
	case 'R':
		modelo.braco.IncRotWrist(3);
		break;
	case 'f':
	case 'F':
		modelo.braco.DecRotWrist(3);
		break;
	case 't':
	case 'T':
		modelo.braco.IncRotClaw(3);
		break;
	case 'g':
	case 'G':
		modelo.braco.DecRotClaw(3);
		break;
	case ' ':
		modelo.braco.ToogleClaw();
		break;
#ifdef POLAR_VIEW
	case 'x':
		((PolarCamara*)estado.cam)->DecAzimuth(2);
		break;
	case 'X':
		((PolarCamara*)estado.cam)->IncAzimuth(2);
		break;
	case 'y':
		((PolarCamara*)estado.cam)->DecElevation(2);
		break;
	case 'Y':
		((PolarCamara*)estado.cam)->IncElevation(2);
		break;
	case 'z':
		((PolarCamara*)estado.cam)->DecDistance(2);
		break;
	case 'Z':
		((PolarCamara*)estado.cam)->IncDistance(2);
		break;
	case 'k':
		((PolarCamara*)estado.cam)->DecTwist(2);
		break;
	case 'K':
		((PolarCamara*)estado.cam)->IncTwist(2);
		break;
#else
	case 'x':
		((EyeCamara*)estado.cam)->DecX(0.1);
		break;
	case 'X':
		((EyeCamara*)estado.cam)->IncX(0.1);
		break;
	case 'y':
		((EyeCamara*)estado.cam)->DecY(0.1);
		break;
	case 'Y':
		((EyeCamara*)estado.cam)->IncY(0.1);
		break;
	case 'z':
		((EyeCamara*)estado.cam)->DecZ(0.1);
		break;
	case 'Z':
		((EyeCamara*)estado.cam)->IncZ(0.1);
		break;
#endif
	case 'c':
	case 'C':
		reset_camera();
		break;	
	case 'o':
	case 'O':
		reset_camera_org();
		break;	
	}
	glutPostRedisplay();
#ifdef _DEBUG 
	estado.cam->DebugOutput();
#endif
}

void init()
{
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_DEPTH_TEST);

	setview();
	reset_camera();
}

void init_menus()
{
	int mnu = glutCreateMenu(menu);
	glutAddMenuEntry("Wireframe",MENU_WIREFRAME);
	glutAddMenuEntry("Solid",MENU_SOLID);
	glutAddMenuEntry("Show Axis",MENU_SHOW_AXIS);
	glutAddMenuEntry("hide Axis",MENU_HIDE_AXIS);
	glutAddMenuEntry("Reset camera position",MENU_RESET_CAMERA);
	glutAddMenuEntry("Put camera at origin",MENU_RESET_CAMERAORG);
	glutAddMenuEntry("Exit",MENU_EXIT);

	glutAttachMenu(GLUT_RIGHT_BUTTON);
}

int main(int argc, CHAR* argv[])
{
	init_estado();

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(500, 500);
	glutInitWindowPosition (100, 100);
	glutCreateWindow("Braco robotico #1");
	init();
	init_menus();

	init_modelo();

	glutKeyboardFunc(keyboard);
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutMainLoop();

	destroy_estado();
	destroy_modelo();

	return 0;
}

