#pragma once
#include <stdio.h>

class Camara
{
public:
	Camara(void);

	virtual void SetView()=0; 
	virtual void DebugOutput()=0;
};

class PolarCamara : public Camara
{
	GLfloat distance;
	GLfloat twist;
	GLfloat elevation;
	GLfloat azimuth;

public:
	PolarCamara(GLfloat distance = 6, GLfloat twist = 0, GLfloat elevation = 0, GLfloat azimuth = 0);

	void SetCamara(GLfloat distance, GLfloat twist, GLfloat elevation, GLfloat azimuth);
	void SetDistance(GLfloat distance)		{ this->distance = distance; }
	void SetTwist(GLfloat twist)			{ this->twist = twist; }
	void SetElevation(GLfloat elevation)	{ this->elevation = elevation; }
	void SetAzimuth(GLfloat azimuth)		{ this->azimuth = azimuth; }

	void IncDistance(GLfloat delta)		{ this->distance += delta; }
	void IncTwist(GLfloat delta)		{ this->twist += delta; if (twist > 360) twist = 0; }
	void IncElevation(GLfloat delta)	{ this->elevation += delta; if (elevation > 360) elevation = 0; }
	void IncAzimuth(GLfloat delta)		{ this->azimuth += delta; if (azimuth > 360) azimuth = 0; }
	void DecDistance(GLfloat delta)		{ this->distance -= delta; }
	void DecTwist(GLfloat delta)		{ this->twist -= delta; if (twist < 0) twist = 360; }
	void DecElevation(GLfloat delta)	{ this->elevation -= delta; if (elevation < 0) elevation = 360; }
	void DecAzimuth(GLfloat delta)		{ this->azimuth -= delta; if (azimuth < 0) azimuth = 360; }

	virtual void SetView();
	void DebugOutput() { printf("Camera: d: %f, e:%f, a:%f, t:%f\n", distance, elevation, azimuth, twist); }
};

class EyeCamara : public Camara
{
	GLfloat x;
	GLfloat y;
	GLfloat z;

	GLfloat cx;
	GLfloat cy;
	GLfloat cz;

	GLfloat ux;
	GLfloat uy;
	GLfloat uz;

public:
	EyeCamara(GLfloat x = 0, GLfloat y = 0, GLfloat z = 6);

	void SetCamara(GLfloat x, GLfloat y, GLfloat z);
	void SetX(GLfloat x)	{ this->x = x; }
	void SetY(GLfloat y)	{ this->y = y; }
	void SetZ(GLfloat z)	{ this->z = z; }

	void IncX(GLfloat delta)	{ this->x += delta; }
	void IncY(GLfloat delta)	{ this->y += delta; }
	void IncZ(GLfloat delta)	{ this->z += delta; }
	void DecX(GLfloat delta)	{ this->x -= delta; }
	void DecY(GLfloat delta)	{ this->y -= delta; }
	void DecZ(GLfloat delta)	{ this->z -= delta; }

	void SetCenter(GLfloat cx, GLfloat cy, GLfloat cz);
	void SetUp(GLfloat ux, GLfloat uy, GLfloat uz);

	virtual void SetView();
	void DebugOutput() { printf("Camera: %f, %f, %f\n", x, y, z); }
};