#include <GL\glut.h>
#include "RobotArm.h"
#include "Util.h"


#define BASE_RADIUS			0.5
#define BASE_HEIGHT			0.25
#define SEG1_WIDTH			0.25
#define SEG1_LENGTH			1.5
#define SEG2_WIDTH			0.25
#define SEG2_LENGTH			1.5
#define WRIST_WIDTH			0.125
#define WRIST_LENGTH		0.25
#define CLAW_BASE_WIDTH		0.25
#define CLAW_BASE_LENGTH	0.01
#define CLAW_WIDTH			0.05
#define CLAW_LENGTH			0.1

void RobotArm::Display(GLenum mode, bool showAxis)
{
	GLfloat mat_1[] = {0.6, 0.6, 0.6, 1.0};
	GLfloat mat_2[] = {1, 1, 0, 1.0};
	GLfloat mat_3[] = {0, 1, 1, 1.0};
	GLfloat mat_4[] = {1, 1, 1, 1.0};
	GLfloat mat_5[] = {1, 0, 1, 1.0};

	glPushMatrix();
		//base
		glRotatef(rotBase, 0, 0, 1);
		if (showAxis)
			drawAxis(0.5);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_1);
		cylinderWithTopAndBottom(mode, BASE_RADIUS, BASE_HEIGHT, 12, 2);

		//segmento 1
		glTranslatef(0, 0, BASE_HEIGHT);
		glRotatef(rotSeg1, 1, 0, 0);
		if (showAxis)
			drawAxis(0.5);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_2);
		box(mode, SEG1_WIDTH, SEG1_LENGTH);

		//segmento 2
		glTranslatef(0, 0, SEG1_LENGTH);
		glRotatef(rotSeg2, 1, 0, 0);
		if (showAxis)
			drawAxis(0.5);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_2);
		box(mode, SEG2_WIDTH, SEG2_LENGTH);

		//pulso
		glTranslatef(0, 0, SEG2_LENGTH);
		glRotatef(rotWrist, 1, 0, 0);
		if (showAxis)
			drawAxis(0.5);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_3);
		box(mode, WRIST_WIDTH, WRIST_LENGTH);

		//garra
		glTranslatef(0, 0, WRIST_LENGTH);
		glRotatef(rotClaw, 0, 0, 1);
		if (showAxis)
			drawAxis(0.5);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_4);
		box(mode, CLAW_BASE_WIDTH, CLAW_BASE_LENGTH);

		// pinas
		glTranslatef(0, 0, CLAW_BASE_LENGTH);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_5);
		float d;
		if (clawOpened)
			d = CLAW_BASE_WIDTH/2;
		else
			d = CLAW_WIDTH/2;

		// pina "direita"
		glPushMatrix();
			glTranslatef(-d, 0, 0);
			box(mode, CLAW_WIDTH, CLAW_LENGTH);
		glPopMatrix();

		// pina "esquerda"
		glPushMatrix();
			glTranslatef(+d, 0, 0);
			box(mode, CLAW_WIDTH, CLAW_LENGTH);
		glPopMatrix();
	glPopMatrix();
}
