#include <math.h>
#include <GL\glut.h>
#include "Util.h"

void polarView(GLdouble distance, GLdouble twist, GLdouble elevation, GLdouble azimuth)
{
	glTranslated(0.0, 0.0, -distance);
	glRotated(+elevation, 1.0, 0.0, 0.0);
	glRotated(azimuth, 0.0, 1.0, 0.0);
	glRotated(-twist, 0.0, 0.0, 1.0);
}

void drawAxis(GLfloat size)
{
	GLfloat mat_r[] = {1.0, 0.0, 0.0, 1.0};
	GLfloat mat_g[] = {0.0, 1.0, 0.0, 1.0};
	GLfloat mat_b[] = {0.0, 0.0, 1.0, 1.0};

	glPushAttrib(GL_LINE_BIT);
	//glLineWidth(2.0);
	glBegin(GL_LINES);
		// XX
		glColor4fv(mat_r);
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_r);
		glNormal3f(0.0, 0.0, 1.0);
		glVertex3f(-size, 0, 0);
		glVertex3f(+size, 0, 0);

		// YY
		glColor4fv(mat_g);
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_g);
		glNormal3f(0.0, 0.0, 1.0);
		glVertex3f(0, -size, 0);
		glVertex3f(0, +size, 0);

		// ZZ
		glColor4fv(mat_b);
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_b);
		glNormal3f(0.0, 0.0, 1.0);
		glVertex3f(0, 0, -size);
		glVertex3f(0, 0, +size);
	glEnd();
	glPopAttrib();
}

int boxL(GLenum mode, GLfloat width, GLfloat height)
{
	GLfloat radius = sqrt(2*width*width)/2;
	return cylinderWithTopAndBottomL(mode, radius, height, 4, 2);
}

int cylinderWithTopAndBottomL(GLenum mode, GLfloat radius, GLfloat height, GLuint slices, GLuint stacks)
{
	int list = glGenLists(2);
	GLUquadricObj* qobj = gluNewQuadric();
	gluQuadricDrawStyle(qobj, mode); //GLU_LINE //GLU_FILL
	gluQuadricNormals(qobj, GLU_SMOOTH);

	glNewList(list+1, GL_COMPILE);
		gluDisk(qobj, 0, radius, slices, stacks);
	glEndList();
	glNewList(list, GL_COMPILE);
		glCallList(list+1);
		gluCylinder(qobj, radius, radius, height, slices, stacks);
		glTranslatef(0, 0, height);
		glCallList(list+1);
	glEndList();
	
	gluDeleteQuadric(qobj);

	return list;
}

void box(GLenum mode, GLfloat width, GLfloat height)
{
	GLfloat radius = sqrt(2*width*width)/2;
	cylinderWithTopAndBottom(mode, radius, height, 4, 2);
}

void cylinderWithTopAndBottom(GLenum mode, GLfloat radius, GLfloat height, GLuint slices, GLuint stacks)
{
	GLUquadricObj* qobj = gluNewQuadric();
	gluQuadricDrawStyle(qobj, mode); //GLU_LINE //GLU_FILL
	gluQuadricNormals(qobj, GLU_SMOOTH);

	glPushMatrix();
		gluDisk(qobj, 0, radius, slices, stacks);
		gluCylinder(qobj, radius, radius, height, slices, stacks);
		glTranslatef(0, 0, height);
		gluDisk(qobj, 0, radius, slices, stacks);
	glPopMatrix();
	
	gluDeleteQuadric(qobj);
}

void cylinder(GLenum mode, GLfloat radius, GLfloat height, GLuint slices, GLuint stacks)
{
	GLUquadricObj* qobj = gluNewQuadric();
	gluQuadricDrawStyle(qobj, mode); //GLU_LINE //GLU_FILL
	gluQuadricNormals(qobj, GLU_SMOOTH);
	gluCylinder(qobj, radius, radius, height, slices, stacks);
	gluDeleteQuadric(qobj);
}

void disk(GLenum mode, GLfloat innerRadius, GLfloat outerRadius, GLuint slices, GLuint stacks)
{
	GLUquadricObj* qobj = gluNewQuadric();
	gluQuadricDrawStyle(qobj, mode); 
	gluQuadricNormals(qobj, GLU_SMOOTH);
	gluDisk(qobj, innerRadius, outerRadius, slices, stacks);
	gluDeleteQuadric(qobj);
}

void partialDisk(GLenum mode, GLfloat innerRadius, GLfloat outerRadius, GLuint slices, GLuint stacks)
{
	GLUquadricObj* qobj = gluNewQuadric();
	gluQuadricDrawStyle(qobj, mode); 
	gluQuadricNormals(qobj, GLU_SMOOTH);
	gluPartialDisk(qobj, innerRadius, outerRadius, slices, stacks, 90, 270);
	gluDeleteQuadric(qobj);
}

void sphere(GLenum mode, GLfloat radius, GLuint slices, GLuint stacks)
{
	GLUquadricObj* qobj = gluNewQuadric();
	gluQuadricDrawStyle(qobj, mode); 
	gluQuadricNormals(qobj, GLU_SMOOTH);
	gluSphere(qobj, radius, slices, stacks);
	gluDeleteQuadric(qobj);
}